/*--------------------------------------------------------------------------
| tput.cpp - Through-Put Testing.  This program is for:

  * This program is used to putz with the FOSSIL driver.

  * Through-Put testing, ability to open up a series of com-ports,
    send and receive data, and track the total and average bytes per second.
    
  * Detect and report Event and Modem Status info inorder to test driver
    functions.

  * Allows turning modem control on/off for first port.

; get command line help
tput

; thoughput test ports 5 through 10.  No handshakes
tput -f5 -l10

; same but with baud = 115200 and RTS/CTS handshaking
tput -f5 -l10 -b115200 -gr1

Copyright 1995-1997 Comtrol Corp.
|--------------------------------------------------------------------------*/
// #include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <ctype.h>
#include <dos.h>

// #define RS485_TEST   

#define getdosticks() (*((unsigned int far *)(0x46cL)))

#define WANT_BASE_TYPES

#ifdef WANT_BASE_TYPES
typedef unsigned char  BYTE;
typedef unsigned short WORD;
typedef unsigned long DWORD;
typedef int BOOL;
#endif

static void putsxy(char *str, int x, int y, char attr);
void test_hook(void);
static int init_fossil(int port);
static DWORD GetCurrentTime(void);
int f_set_com_options(int port,
                      DWORD baudrate,
                      int Parity,
                      int StopBits,
                      int ByteSize);
int test_num1(void);
int test_num2(void);

/*--- kom.h ---*/
class Kom {
  public:

  ~Kom();
  Kom(int port);
  int open(void);
  int close(void);
  int get_state(void);
  void print_changes(int stat);
  int get_status(int *out_free, int *in_cnt, int *ret_stat);
  int put(unsigned char *str, int cnt);
  int put_file(unsigned char *str, int cnt);
  int get(unsigned char *str, int *ret_cnt, int want);
  int  get_file(unsigned char *str, int *ret_cnt, int want);

  // following is for keeping track of all kom ports in/out bytes.
  static double GrandInTotal1;
  static double GrandInTotal2;
  static double GrandOutTotal1;
  static double GrandOutTotal2;
  int  data_errs;
  unsigned char last_char;
  unsigned char dummy;
//  char name_str[60];
  int opened;
  int port_num;
  int   status;  // non-zero error code if something wrong
  DWORD OutTotal;
  DWORD InTotal;
  FILE *hWrFile;    // for file write.
  FILE *hRdFile;    // for file read.
  int ver_tot;  // needed to keep track of bytes to send with verification.

  WORD OldModemStatus;
  WORD stat;     // bios 03 stat AX value
  WORD old_stat; // last bios 03 stat AX value
  WORD display_stat; // used to detect change in stat
};
/*--- end kom.h ---*/

// Misc settings
int loop_back = 0;
int our_queue_out_size = 0;
int our_queue_in_size = 0;
int wait_for_keypress = 0;
int our_debug = 0;
int verbose_level = 0;
int skip_recfile = 0;
int use_default_shakes = 0;
static char *tst_str = {"abcdefghijklmnopqrstuvwxyz\x0d\x0a"};
int tst_str_len = strlen(tst_str);
int chunk_size = 128 - (128 % tst_str_len);  // make even multiple of str
int use_rec_data_verification = 0;
static unsigned char data_rec_char[15] = {0,0,  0x55, 0x40, 0xbf, 0xff, 0,
                                         0x01, 0x02,0x04,0x08,0x10, 0x20};
int use_input_flag = 1;
int use_output_flag = 1;
int test_sec = 0;
int test_reps = 1;
int test_num = 1;
int timeout_constant = 0;
int timeout_multiplier = 0;
int use_rts_shakes = 0;
int use_dtr_shakes = 0;
int use_xon_shakes = 0;
int use_file_rec = 0;
int use_file_send = 0;
Kom *Koms[160];
int show_incoming_data = 0;
DWORD BaudRate = 38400;
int Parity = 0;
int ByteSize = 8;
int StopBits = 0;  // 1, 1.5, 2

double Kom::GrandInTotal1 = 0;
double Kom::GrandInTotal2 = 0;
double Kom::GrandOutTotal1 = 0;
double Kom::GrandOutTotal2 = 0;

 int first_port = 5;
 int last_port = 0;

 unsigned char *inbuf;
 unsigned char *outbuf;
 int num_ports;
 char tmpstr[80];
 int passes = 0;
 DWORD TotalErrors = 0;
 int in_q;
 int out_q;
 int last_err;

 int show_modem_status = 0;
 int show_events = 0;
 int hook_it = 0;
 WORD our_ds;

#include "fu.c"

#define MODEM_STAT (MDMSTAT_DCD | MDMSTAT_CTS | MDMSTAT_DSR | MDMSTAT_RI)
#define MODEM_ERRS (LNSTAT_OVRRUN | LNSTAT_PARITY | LNSTAT_FRAMING | \
                    LNSTAT_TIMEOUT)
#define MODEM_MISC (LNSTAT_DATARDY | LNSTAT_BREAK | LNSTAT_THRE | LNSTAT_TE)

FossInfo fi;  // global fossil info struct

/*--------------------------------------------------------------------------
| main -
|--------------------------------------------------------------------------*/
int main(int argc, char **argv)
{
 int stat, i, j;
 int alloc_size;
 int help_flag = 0;
 char *buf;

  BaudRate = 9600;
  Parity = 0;
  StopBits = 0;  // 1 stop bit
  ByteSize = 8;  // byte size

  for (i=1; i<argc; i++)
  {
    buf = argv[i];
    if ((*buf == '-') || (*buf == '/'))
    {
      ++buf;
      switch(toupper(*buf++))
      {
        case 'A':
          switch(toupper(*buf++))
          {
            case 'I': use_input_flag = atoi(buf); break;
            case 'O': use_output_flag = atoi(buf); break;
            case 'D' : use_rec_data_verification = atoi(buf); break;
            case 'S' : show_incoming_data = atoi(buf); break;
          }
        break;

        case 'B': BaudRate = atol(buf); break;

        case 'E':   // -EE -EM -EH
          switch(toupper(*buf++))
          {
            case 'E': show_events = atoi(buf); break;
            case 'M': show_modem_status = atoi(buf); break;
            case 'H': hook_it = 1;
               // save our DS, so we have our bearings at irq time
               ASM { mov our_ds, DS}
            break;
          }
        break;

        case 'F':
          first_port = atoi(buf);
          if (last_port < first_port) last_port = first_port;
        break;

        case 'G':   // -G 0/1 Use Hardware Handshakes
          switch(toupper(*buf++))
          {
            case 'D': use_default_shakes = 1; break;
            case 'R': use_rts_shakes = atoi(buf); break;
            case 'X': use_xon_shakes = atoi(buf); break;
          }
        break;

        case 'L': last_port = atoi(buf); break;

        case 'H':  // help
        case '?':  // help
          help_flag = 1;
        break;

        case 'N':  // bits
          ByteSize = atoi(buf);
        break;

        case 'P':   // -P {0/n/o/e/m/s}
          switch(toupper(*buf++))
          {
            case '0':
            case 'N':  // none
              Parity = 0;
            break;
            case 'O':  // odd
              Parity = 1;
            break;
            case 'E':  // even
              Parity = 2;
            break;
            case 'M':  // mark
              Parity = 3;
            break;
            case 'S':  // space
              Parity = 4;
            break;
          }
        break;

        case 'S':  // stopbits
          int stop_bits;
          stop_bits = atoi(buf);
          if (stop_bits == 1)
            StopBits = 0;  // 1 stop bit
          else if (stop_bits == 2)
            StopBits = 2;  // 2 stop bit
          else if (stop_bits == 15)
            StopBits = 1;  // 1.5 stop bit
          else
          {
            printf("warning: %d stop bits not supported.\n",stop_bits);
          }
        break;

        case 'T':
          switch(toupper(*buf++))
          {
            case 'T' : test_sec = atoi(buf);  break;
            case 'L' : loop_back = atoi(buf); break;
            case 'R' : test_reps = atoi(buf); break;
            case 'N' : test_num  = atoi(buf); break;
            default:
              printf("Bad queue size option\n");
            break;
          }
        break;

        case 'Y':
          switch(toupper(*buf++))
          {
            case 'S' : use_file_send = atoi(buf); break;
            case 'R' : use_file_rec  = atoi(buf); break;
          }
        break;

        case 'Z': chunk_size = atoi(buf); break;

        default:
          printf("option %c not supported\n", *(buf-1));
          help_flag = 1;
        break;
      }
    }
    else
    {
      printf("Expecting Switch -#####\n");
    }
  }
  
  if ((help_flag) || (argc == 1))
  {
    printf("TPUT will test throughput on a range of FOSSIL com-ports.\n");
    printf("Comtrol Corp., copyright 1995\n");
    printf("Use>TPUT -options\n");
    printf("Where options:\n");
    printf("-AO# = 0/1, 0=Output Off, 1=Output On, def=1\n");
    printf("-AI# = 0/1, 0=Input Off, 1=Input On, def=1\n");
    printf("-AD1 = Verify Data Integrity\n");
    printf("-AS1 = Show incoming data\n");
    printf("-B###### = baud rate\n");
    printf("-EM# = Show Modem Status,  def=0\n");
    printf("-EE# = Show Event Status,  def=0\n");
    printf("-F###### = first com port number\n");
    printf("-GR# = 0/1 use RTS/CTS hardware handshakes, def=0=off\n");
    printf("-GX# = 0/1 use XON/XOFF software handshakes, def=0=off\n");
    printf("-GD  = do not setup handshaking(use drivers defaults.)\n");
    printf("-L###### = last  com port number\n");
    printf("-N###### = Byte Size 7/8, def=8\n");
    printf("-H = This help file.\n");
    printf("press any key to continue.\n");
    getch();
    printf("-P###### = parity (0/n/o/e/m/s), def=none\n");
    printf("-S###### = stopbits (1/2/15)\n");
    printf("-TN###### = Test Number, def=1(throughput testing)\n");
    printf("            2=Open/Close repition test\n");
    printf("-TL1 = Loop back ports in hardware\n");
    printf("-TT###### = Time to run test(seconds)(0=forever) def=0\n");
    printf("-TR###### = Test Repitions, def=1\n");
    printf("-YR# = write receive data to file: KOM###.REC, def=0\n");
    printf("-YS# = read data to send from file: KOM###.SND, def=0\n");
    printf("-Z###### = read/write chunk size(def 128bytes)\n");
    printf("keys for first port: r-RTS off, R-RTS on, d-DTR off, D-DTR on.\n");
    printf("t-send test string. B-BREAK on, b-BREAK off. Space-Pause\n");
    printf("EXAMPLE: tput -f5 -l10\n");
    return 1;
  }

  init_fossil(first_port-1);  // get in,out buffer sizes.

  printf("Input Buf Size: %d, Output Buf Size:%d\n", our_queue_out_size,
                               our_queue_in_size);
  printf("Fossil Driver %s\n Version: %d.%d\n",fi.ident, fi.majver,fi.minver);
  printf("Fossil ID structure size:%d, Baud:%d\n",fi.strsiz,fi.baud);

  if (last_port < first_port) last_port = first_port;
  if (use_rec_data_verification)
  {
//    chunk_size = 0x400;
    chunk_size = 0x80;  // buffers very small in fossil drivers.
  }

  alloc_size = chunk_size;

  if (alloc_size < 0x1000)  // data integrity chk needs at least 256,
    alloc_size = 0x1000;    // buf size, even if chunk is smaller.
  
  inbuf = (unsigned char *) calloc(alloc_size+20,1);
  outbuf = (unsigned char *) calloc(alloc_size+20,1);

  if (hook_it)
    FOSSset_hook(test_hook);

  for (i=0; i<test_reps; i++)
  {
    switch(test_num)
    {
      case 1: stat = test_num1(); break;
      case 2: stat = test_num2(); break;
    }
    if (stat != 0)
    {
      if (hook_it)
        FOSSset_hook(NULL);
      return 1;
    }
  }
  if (hook_it)
    FOSSset_hook(NULL);

  return 0;
}

/*----------------------------------------------------------------------------
| putsxy -
|----------------------------------------------------------------------------*/
static void putsxy(char *str, int x, int y, char attr)
{
 unsigned char far *scr;
 int offset = (160 * y) + (2 * x);
 int j;

  scr = (char far *) 0xB8000000L;

  scr += offset;
  while (*str)
  {
    *scr++ = *str++;
    *scr++ = attr;
  }
}

/*--------------------------------------------------------------------
| test_hook -
|---------------------------------------------------------------------*/
void test_hook(void)
{
 static unsigned int i;
 static char teststr[6];

 // ASM {mov AX, DGROUP}
  ASM {mov DS, our_ds}
  ASM {mov ES, our_ds}
  teststr[0] = 't';
  teststr[1] = 'e';
  teststr[2] = 's';
  teststr[3] = 't';
  teststr[4] = 0;
  //putsxy("test", i % 40, i % 24, 0x50);
  putsxy(teststr, i % 40, i % 24, (i & 0xff));
  ++i;
}

/*--------------------------------------------------------------------
| test_num1 - Do through-put testing on all ports.
|---------------------------------------------------------------------*/
int test_num1(void)
{
  int stat, cnt, i, j, k, c;
  j = 0;

  if (use_rec_data_verification == 0)
  {
    for (i=0; i<chunk_size; i++)
    {
      outbuf[i] = tst_str[j++];
      if (j >= tst_str_len)
        j = 0;
    }
  }
  else if (use_rec_data_verification == 1)
  {
    for (i=0; i<600; i++)
      outbuf[i] = (unsigned char) i;
             // can't include 0x11 / 0x13 (XON/XOFF) in stream
    if ((use_xon_shakes) && (outbuf[i] < 0x20))
      outbuf[i] |= 0x020;
  }
  else
  {
    for (i=0; i<600; i++)
      outbuf[i] = data_rec_char[use_rec_data_verification];
  }

  printf("\x0d Test 1, ----Throughput Testing---- ");
  for (i=first_port; i<=last_port; i++)
  {
    printf("\x0dOpening Port: COM%d. ",Koms[i]->port_num+1);

    Koms[i] = new Kom(i);
    if (Koms[i]->open() != 0)
    {
      printf("Opening COM%d failed\n",Koms[i]->port_num+1);
      return 1;
    }

    if (loop_back)
      FOSSloopback(Koms[i]->port_num, 1);

    if (ByteSize != 0)
    {
      stat = f_set_com_options(Koms[i]->port_num, BaudRate, Parity, StopBits, ByteSize);
      if (stat != 0)
      {
        printf("Error Setting Options.");
        return 3;
      }
    }
  }

  printf("\x0d Start Testing  ");   // clear our status line
  printf("                                           ");  // clear line

DWORD t0,t1;
DWORD time_rate, old_time_rate, timestart, timecur, cur_sec;
// DWORD oldtimecur = 0xff;
int done = 0;
int chg_flags;
char tmpstr[5];
int all_ports_done = 0;

 TotalErrors = 0;
 Kom::GrandInTotal1 = 0;
 Kom::GrandInTotal2 = 0;
 Kom::GrandOutTotal1 = 0;
 Kom::GrandOutTotal2 = 0;

  num_ports = (last_port - first_port) + 1;
  timestart = GetCurrentTime();
  timecur = GetCurrentTime() - timestart;
  old_time_rate = timecur;
  t1 = timecur;
  t0 = timecur;
  cur_sec = 0;

  while (!done)                                            // main loop
  {
//    if (timecur != oldtimecur)
    {
//      oldtimecur = timecur;
    for (i=first_port; i<=last_port; i++)
    {
      ++passes;
      Koms[i]->get_status(&out_q, &in_q, &stat);
      if (stat != Koms[i]->old_stat)
      {
        Koms[i]->old_stat = stat;
        Koms[i]->print_changes(stat);
        if (stat & MODEM_ERRS)
        {
          last_err = (stat & MODEM_ERRS);
          ++TotalErrors;
        }
      }

#ifdef RS485_TEST   
      if ((use_output_flag) && (out_q > chunk_size) && (i == first_port))
#else
      if ((use_output_flag) && (out_q > chunk_size))
#endif
      {
        if (use_file_send)
        {
          Koms[i]->get_file(inbuf, &j, chunk_size);
          if (j > 0)
          {
            Koms[i]->put(inbuf, chunk_size);
            all_ports_done = 0;
          }
          else
          {
            ++all_ports_done;
            if (all_ports_done > 1000)
              done = 1;  // end test
          }
        }
        else
        {
          if (use_rec_data_verification)
          {
            Koms[i]->put(&outbuf[Koms[i]->ver_tot], chunk_size);
            Koms[i]->ver_tot = (Koms[i]->ver_tot + chunk_size) % 256;
          }
          else
            Koms[i]->put(outbuf, chunk_size);
        }
      }

      if ((use_input_flag) && (in_q > 0))
      {
        if (in_q > chunk_size)
          in_q = chunk_size;
        Koms[i]->get(inbuf, &j, in_q);
        if (use_file_rec)
          Koms[i]->put_file(inbuf, j);
        if (use_rec_data_verification)
        {
          for (k=0; k<j; k++)
          {
            c = inbuf[k];
            if (use_rec_data_verification == 1)
            {
              if (c != (unsigned char)(Koms[i]->last_char+1))
              {
                 // special handling for xon/xoff
                if ( (!use_xon_shakes) ||
             ((Koms[i]->last_char != 0xff) && (Koms[i]->last_char != 0x3f))
                   )
                printf("\x0d Data[%d] Errs: %d COM:%d Want:%x Got:%x. \n",
                        k,++Koms[i]->data_errs, i, Koms[i]->last_char+1, c);
              }
              Koms[i]->last_char = c;
            }
            else
            {
              if (c != data_rec_char[use_rec_data_verification])
                printf("\x0d Data[%d] Errs: %d COM:%d Want:%x Got:%x. \n",
                        k,++Koms[i]->data_errs, i, data_rec_char[use_rec_data_verification], c);
            }
          }
        }
        if (show_incoming_data)
        {
          inbuf[j] = 0;
          printf("%s",inbuf);
        }
      }
    }
    if (kbhit())
    {
      c = getch();
      switch (c)
      { 
        case 'A':
        case 'a':
          printf("\nset dcb.\n");
        break;
        case 'S':
        case 's':
          printf("\nkey to send:");
          tmpstr[0] = getch();
          Koms[first_port]->put((unsigned char *)tmpstr, 1);
          printf("\n");
        break;
        case 0x1b:
          done = 2;
        break;
        case 'c':
        case 'C':
          for (i=first_port; i<=last_port; i++)
          {
            printf("PORT %4d, SENT=%4ld, REC=%4ld\n",
                   i, Koms[i]->OutTotal,  Koms[i]->InTotal);
            if ((i%16)==15) getch();
          }
        break;
        case 't':
        case 'T':
          printf("<");
          Koms[first_port]->put((unsigned char *)"SEND OUT A STRING\x0d\x0a", 0);
          printf(">\n");
        break;

        case 'u':
        case 'U':
          printf("{");
          Koms[first_port+1]->put((unsigned char *)"REC   IN A STRING\x0d\x0a", 0);
          printf("}\n");
        break;

        case 'R': FOSSrts(Koms[first_port]->port_num,1); break;
        case 'r': FOSSrts(Koms[first_port]->port_num,0); break;
        case 'D': FOSSdtr(Koms[first_port]->port_num,1); break;
        case 'd': FOSSdtr(Koms[first_port]->port_num,0); break;
        case ' ':
          printf("\npress a key...");
          getch();  printf("\n");
        break;
        case 'B': FOSSbreak(Koms[first_port]->port_num,1); break;
        case 'b': FOSSbreak(Koms[first_port]->port_num,0); break;
      }
    }
    }  // timechange

    timecur = GetCurrentTime() - timestart;

    t1 = timecur;
    if ((t1 - t0) > 2000)
    {
      cur_sec += (t1 - t0)/1000;
      t0 = t1;
      time_rate = timecur - old_time_rate;

      if (show_incoming_data)
        printf("\n");
      if (time_rate != 0)
        printf("\x0d%4.2lfk sent(%4.2lfk/s), %4.2lfk rec(%4.2lfk/s) Errs:%u %x ",
             Kom::GrandOutTotal1/1000,
             Kom::GrandOutTotal2 / (time_rate * num_ports),
             Kom::GrandInTotal1/1000,
             Kom::GrandInTotal2 / (time_rate * num_ports),
             TotalErrors,
             last_err);

      if (time_rate > 20000)
      {
        Kom::GrandOutTotal2 = 0;
        Kom::GrandInTotal2 = 0;
        old_time_rate = timecur;
      }

      if ((test_sec > 0) && (cur_sec > test_sec))
        done = 1;
    }  // show

  } // !done
  
  for (i=first_port; i<=last_port; i++)
  {
    if (loop_back)
      FOSSloopback(Koms[i]->port_num, 0);

    printf("\x0d Closing Port: COM%d. ",Koms[i]->port_num+1);
    Koms[i]->~Kom();
  }

  printf("\n");

  if (done == 1)  // timeout
    return 0;

  return 1;  // keypress or error.
}

/*--------------------------------------------------------------------
| test_num2 - Open & Close test.
|---------------------------------------------------------------------*/
int test_num2(void)
{
  DWORD timestart, timecur, cur_sec;
  int stat, cnt, i;
  int times = 0;
  int done = 0;
//  j = 0;

  printf("\x0d Test 1, ----Open/Close Testing---- ");
  for (i=first_port; i<=last_port; i++)
  {
    Koms[i] = new Kom(i);
  }

  printf("\x0d Start Testing  ");   // clear our status line
  printf("                                           ");  // clear line

  timestart = GetCurrentTime();
  while (!done)
  {
    printf("\x0d Opens & Closes %d     ",times++);
    for (i=first_port; i<=last_port; i++)
    {
      if (Koms[i]->open() != 0)
      {
        printf("\nfatal error\n");
        return 1;
      }
    }
    for (i=first_port; i<=last_port; i++)
    {
      Koms[i]->close();
    }
    timecur = GetCurrentTime() - timestart;
    cur_sec = timecur/1000;
    if ((test_sec > 0) && (cur_sec > test_sec))
      done = 1;

    if (kbhit())
    {
      done = 2;
      getch();
    }
  }
  for (i=first_port; i<=last_port; i++)
  {
    printf("\x0d Closing Port: COM%d. ",Koms[i]->port_num+1);
    Koms[i]->~Kom();
  }
  printf("\n");

  if (done == 1)  // timeout
    return 0;

  return 1;  // keypress or error.
}

/*--------------------------------------------------------------------
| Kom::print_changes -
|---------------------------------------------------------------------*/
void Kom::print_changes(int stat)
{
  if (show_modem_status)
  {
    if ((stat & MODEM_STAT) != (display_stat & MODEM_STAT))
    {
      printf("\x0d MODEM STATE: %s %s %s %s",
     (stat & MDMSTAT_CTS)   ? " CTS"  : " cts"  ,
     (stat & MDMSTAT_DSR)   ? " DSR"  : " dsr"  ,
     (stat & MDMSTAT_RI)    ? " RING" : " ring" ,
     (stat & MDMSTAT_DSR)   ? " RLSD" : " rlsd" );
     printf("    \n");
    }
  }

  if ((stat & MODEM_ERRS) != 0)
  {
    printf("\x0d ERROR: ");
    if (stat & LNSTAT_OVRRUN ) printf("OVRRUN ");
    if (stat & LNSTAT_PARITY ) printf("PARITY ");
    if (stat & LNSTAT_FRAMING) printf("FRAMING ");
    if (stat & LNSTAT_TIMEOUT) printf("TIMEOUT ");
    printf(" COM%d \n", port_num+1);
  }

  if (show_events)
  {
    if ((stat & MODEM_MISC) != (display_stat & MODEM_MISC))
    {
      printf("\x0d ERROR: ");
      if (stat & LNSTAT_DATARDY) printf("DATARDY ");
      if (stat & LNSTAT_BREAK  ) printf("BREAK ");
      if (stat & LNSTAT_THRE   ) printf("TXHREADY ");
      if (stat & LNSTAT_TE     ) printf("TXEMPTY ");
      printf(" COM%d \n", port_num+1);
    }
  }
  display_stat = stat;
  display_stat &= ~MODEM_ERRS;
}

/*--------------------------------------------------------------------
| f_com_options -
|---------------------------------------------------------------------*/
int f_set_com_options(int port,
                      DWORD BaudRate,
                      int Parity,
                      int StopBits,
                      int ByteSize)
{
 WORD val, bval, bsize, stops;

  switch (BaudRate)
  {
    case 110:   bval = 0; break;
    case 150:   bval = 1; break;
    case 300:   bval = 2; break;
    case 600:   bval = 3; break;
    case 1200:  bval = 4; break;
    case 2400:  bval = 5; break;
    case 4800:  bval = 6; break;
    case 9600:  bval = 7; break;
    case 19200: bval = 8; break;
    case 28800: bval = 0x80; break;
    case 38400: bval = 0x81; break;
    case 57600: bval = 0x82; break;
    case 76800: bval = 0x83; break;
    case 115200:bval = 0x84; break;
    case 230400:bval = 13; break;
    case 460800:bval = 14; break;
    case 921600:bval = 15; break;
    default  :  bval = 7; break;
  }

  switch (ByteSize)
  {
    case 5:
    case 6:
    case 7: bsize = 2; break;  // 7
    case 8: bsize = 3; break;  // 8
  }

  FOSSbaudExt(port, bval, bsize, Parity, StopBits);

  if (use_default_shakes == 0)
  {
    val = 0;
    if (use_rts_shakes) val = 2;
    if (use_xon_shakes) val |= 9;

    FOSSflowCtrl(port, val);
  }

  return 0;
}


/*-------- kom class implementation -----*/
/*--------------------------------------------------------------------------
| Kom::Kom - constructor
|--------------------------------------------------------------------------*/
Kom::Kom(int port)
{
  ver_tot = 0;
  data_errs = 0;
  last_char = 0xff;
  OutTotal = 0;
  InTotal = 0;
  hWrFile = NULL;
  hRdFile = NULL;
  port_num  = port-1;
}

/*--------------------------------------------------------------------------
| Kom::~Kom - destructor
|--------------------------------------------------------------------------*/
Kom::~Kom(void)
{
  close();
}

/*--------------------------------------------------------------------------
| Kom::close -
|--------------------------------------------------------------------------*/
int Kom::close(void)
{
  if (opened)
  {
    FOSSflushtx(port_num);
    FOSSclose(port_num);
  }
  if (hWrFile != NULL)
    fclose(hWrFile);
  if (hRdFile != NULL)
    fclose(hRdFile);
  return 0;
}

/*--------------------------------------------------------------------------
| Kom::open -
|--------------------------------------------------------------------------*/
int Kom::open(void)
{
 WORD fstat;

  fstat = FOSSinit(port_num);
  if (fstat == 0x1954)
  {
    opened = 1;
    return 0; // ok
  }
  return 1; // err
}

/*--------------------------------------------------------------------
| Kom::put -
|---------------------------------------------------------------------*/
int Kom::put(unsigned char *str, int cnt)
{
 int stat,j;
 WORD byteswritten;

  if (!opened) return 1;

  if (cnt == 0) cnt = strlen((char *)str);

  j = FOSSwriteblk(port_num, cnt, str);
  OutTotal += j;  // assume it will be sent
  GrandOutTotal1 += j;  // assume it will be sent
  GrandOutTotal2 += j;  // assume it will be sent

  if (j != cnt)
  {
    printf("Write only %d, wanted %d\n",j,cnt);
  }
  return 0;
}

/*--------------------------------------------------------------------
| Kom::put_file - put received data into COMXXX.REC
|---------------------------------------------------------------------*/
int Kom::put_file(unsigned char *str, int cnt)
{
 int stat,j;
 DWORD dwBytesWritten;
 char tmpstr[20];
 DWORD pos;

  if (hWrFile == NULL)  // need to open the file
  {                          // pass up \\.\ part of string
    sprintf(tmpstr, "KOM%d.REC",port_num+1);
    hWrFile = fopen(tmpstr,"wb");
    if (hWrFile == NULL)
    {
      printf("Could not open file!\n");
      return 3;   // err
    }
  }

  if (cnt == 0) return 0;  // ok

  j = fwrite(str, 1, cnt, hWrFile);

  if (j != cnt)
  {
    printf("Write only %d, wanted %d\n",j,cnt);
    return 2;  //err
  }
  return 0;
}

/*--------------------------------------------------------------------
| Kom::get_file - get data from COM###.SND to send.
|---------------------------------------------------------------------*/
int Kom::get_file(unsigned char *str, int *ret_cnt, int want)
{
 int stat,j;
 char tmpstr[20];
 DWORD pos;

  if (hRdFile == NULL)  // need to open the file
  {                          // pass up \\.\ part of string
    sprintf(tmpstr, "KOM%d.SND",port_num+1);
    hRdFile = fopen(tmpstr,"rb");
    if (hRdFile == NULL)
    {
      printf("Could not open RD file!\n");
      return 3;   // err
    }
  }
  if (want == 0) return 0;  // ok

  j = fread(str, 1, want, hRdFile);
  *ret_cnt = j;

  return 0;
}

/*--------------------------------------------------------------------
| Kom::get - get a number of bytes from the com-port.
|---------------------------------------------------------------------*/
int Kom::get(unsigned char *str, int *ret_cnt, int want)
{

 int BytesRead;

  if (!opened) return 1;

  BytesRead = FOSSreadblk(port_num, want, str);
  if (BytesRead != want)
  {
    printf("Read only %d, wanted %d\n", BytesRead, want);
  }
  InTotal += BytesRead;
  GrandInTotal1 += BytesRead;
  GrandInTotal2 += BytesRead;
  *ret_cnt = BytesRead;
  return 0;
}

/*--------------------------------------------------------------------
| Kom::get_state - Check for Events and ModemStatus.
|---------------------------------------------------------------------*/
int Kom::get_state(void)
{
  stat = FOSSstatus(port_num);
  if (stat == old_stat)
    return 0; // no changes to status

  old_stat = stat;
  return 1;  // something changed
}

/*--------------------------------------------------------------------
| Kom::get_status - Return Com Port Out/In queue count, as well as
   returning any present Com errors(Causes reset of errors also.)
|---------------------------------------------------------------------*/
int Kom::get_status(int *out_free, int *in_cnt, int *ret_stat)
{
 FossInfo fi;

  if (!opened) return 1;
  FOSSgetinfo(port_num, &fi);

  *ret_stat = FOSSstatus(port_num);
  *in_cnt  =  fi.ibufr - fi.ifree;
  *out_free = fi.ofree;

  return 0;
}

/*--------------------------------------------------------------------
| init_fossil -
|---------------------------------------------------------------------*/
static int init_fossil(int port)
{
 WORD fstat;

  fstat = FOSSinit(port);
  if (fstat == 0x1954)
  {
    FOSSgetinfo(port, &fi);
    our_queue_out_size = fi.ibufr;
    our_queue_in_size = fi.obufr;
    FOSSclose(port);
  }
  else
  {
    printf("Error Opening Fossil!, NOT 1954H\n");
    our_queue_out_size = 0;
    our_queue_in_size = 0;
  }

  return 0;
}

/*--------------------------------------------------------------------
| GetCurrentTime -
|---------------------------------------------------------------------*/
static DWORD GetCurrentTime(void)
{
 DWORD t;
  t = getdosticks();
  t = (t * 1000)/18;
  return t;
}
